----------------------------------
-- CREATING FUNCTIONS
----------------------------------

-- A FUNCTION IS LIKE A PROCEDURE, WITH A FEW DIFFERENCES:
-- 1) MUST RETURN A VALUE
-- 2) ONLY ACCEPT IN PARAMETERS
-- 3) CAN BE USED IN SQL CLAUSES (WHERE, ORDER BY, GROUP BY, ETC)
-- 4) CANNOT CONTAIN CERTAIN SQL STATEMENTS


-- SYNTAX:

CREATE [OR REPLACE] FUNCTION <name>
[ (par1 type1, par2 type2,...) ]
RETURN <datatype>
IS | AS
BEGIN
  ...
EXCEPTION
  ...
END <name>;

-- USE SHOW ERRORS IN SQL*PLUS



-- EXAMPLE 1: IN PARAMETER

CREATE OR REPLACE FUNCTION HR.F_SALARY( p_id IN EMPLOYEES.EMPLOYEE_ID%TYPE) 
RETURN NUMBER
IS
  v_sal HR.EMPLOYEES.SALARY%TYPE;
BEGIN
  SELECT SALARY
  INTO v_sal
  FROM HR.EMPLOYEES
  WHERE EMPLOYEE_ID = p_id;

  RETURN v_sal*1.1;

END HR.F_SALARY;

-- EXECUTE 1:

SELECT EMPLOYEE_ID
       ,SALARY
      ,F_SALARY(EMPLOYEE_ID)
FROM HR.EMPLOYEES;

-- EXECUTE 2:

DECLARE
  sal NUMBER;
BEGIN
  sal := F_SALARY(176);
  
  DBMS_OUTPUT.PUT_LINE('Salary: ' || TO_CHAR(sal) );
END;


-- SQL RESTRICTIONS:
-- ACCEPT AND RETURN ONLY SQL DATATYPES
-- CANNOT CONTAINT DML STATEMENTS
-- CANNOT FINISH TRANSACTIONS


-- WRONG EXAMPLE 2:

CREATE OR REPLACE FUNCTION HR.F_DELETE( p_id IN EMPLOYEES.EMPLOYEE_ID%TYPE)
RETURN NUMBER
IS
BEGIN
  DELETE HR.EMPLOYEES
  WHERE EMPLOYEE_ID = p_id;
  
  COMMIT;
  
  RETURN TO_NUMBER(SQL%ROWCOUNT);

END HR.F_DELETE;

-- EXECUTION

SELECT LAST_NAME
       ,F_DELETE(EMPLOYEE_ID)
FROM HR.EMPLOYEES

-- TO REMOVE A FUNCTIONS:

DROP FUNCTION HR.F_SALARY;
DROP FUNCTION HR.F_DELETE;

-- COMPARING FUNCTIONS AND PROCEDURES

-----------------------------     ----------------------------
PROCEDURES                        FUNCTIONS
-----------------------------     ----------------------------
EXECUTE AS PL/SQL STATEMENT       INVOKE AS PART OT STATEMENT
IN, OUT AND IN/OUT PARAMETERS     ONLY IN PARAMETERS
NO RETURN VALUES                  MUST RETURN A VALUE
0, 1 OR MORE RETURN VALUES        SINGLE RETURN VALUE
NO SQL RESTRICTIONS               FEW SQL RESTRICTIONS